<?php

namespace LMS\Libraries\Base;

use LMS\Libraries\Servicelocator;

class Widget
{	
	protected $params;
	
	protected $name;
	
	protected $path = array('template'=>array());
	
	protected $forms;
	
	protected $widgets = array();
	
	protected $view;
	
	protected $config;
	
	protected $tmp = 'default';
	
	public function __construct($params = array()) 
	{
		ServiceLocator::load('lib.base');
		$this->assets = ServiceLocator::get('assets');		

		$this->widgetDir = empty($params['widgetDir'])?JPATH_COMPONENT.'/widgets/':$params['widgetDir'];
		$this->widgetDir .= $this->getName();
		
		$this->addTemplatePath(JPATH_COMPONENT.'/widgets/'.$this->getName().'/tmpl');				
	}
	
	protected function getTemplate() {		
		return $this->tmp;
	}
	
	protected function setTemplate($tmp) {
		$this->tmp = $tmp;		
	}
	
	public function render()
	{		
		$content = '';				
		
		if($path = \JPath::find($this->path['template'], $this->getTemplate().'.php') ){			
			ob_start();
			require $path;
			$content = ob_get_contents();
			ob_end_clean();
		}
		
		return $content;
	}
	
	protected function getName() 
	{
		if (empty($this->name))
		{	
			$classArr = explode('\\', get_class($this));
			$className = array_pop($classArr); //cat class name from fullpath namespace
			$this->name = strtolower($className);		
		}

		return $this->name;	
	}
	
	protected function addTemplatePath($path) 
	{		
		settype($path, 'array');
	
		foreach ($path as $dir)
		{			
			$dir = trim($dir);
			
			if (substr($dir, -1) != DIRECTORY_SEPARATOR)
			{				
				$dir .= DIRECTORY_SEPARATOR;
			}

			// Add to the top of the search dirs
			array_unshift($this->path['template'], $dir);
		}
	}
	
	protected function loadTemplate($name)
	{
		if($path = \JPath::find($this->path['template'], $name . '.php')) {
			require $path;
		}
	}
	
	protected function init() {}
	
	protected function widget($name, $params = array(), $uniqueKey = '')
	{	
		$key = empty($uniqueKey)?$name:$name.$uniqueKey;
		
		if(isset($this->widgets[$key])) {	
			return $this->widgets[$key];
		} else {			
			$params['context'] = $key;			
			
			$className = '\\LMS\\Widgets\\'.ucfirst(strtolower($name));		
			$class = new \ReflectionClass($className);			
			
			$this->widgets[$key] = $class->newInstanceArgs(array($params));			
			return $this->widgets[$key];
		}
	}
	
	protected function getForm($name, $params = array()) 
	{		
		$loadParams = array_merge(array('control' => 'jform', 'load_data' => true), $params);
		
		return  $this->loadForm('com_joomla_lms.'.$name, $name, $loadParams);	
	}
	
	protected function loadFormData()
	{
		return array();
	}
	
	protected function loadForm($name, $source = null, $options = array(), $clear = false, $xpath = false)
	{		
		$options['control'] = \JArrayHelper::getValue($options, 'control', false);
	
		$hash = md5($source . serialize($options));


		if (isset($this->forms[$hash]) && !$clear)
		{
			return $this->forms[$hash];
		}

		\JForm::addFormPath(JPATH_COMPONENT . '/libraries' . '/form' . '/' . '/forms');
		\JForm::addFieldPath(JPATH_COMPONENT . '/libraries' . '/form' . '/' . '/fields');		
		\JForm::addFormPath($this->widgetDir . '/forms');
		\JForm::addFieldPath($this->widgetDir . '/fields');		

		try
		{
			$form = \JForm::getInstance($name, $source, $options, false, $xpath);			

			if (isset($options['load_data']) && $options['load_data'])
			{				
				$data = $this->loadFormData();
			}
			else
			{
				$data = array();
			}
			
			$this->preprocessForm($form, $data);

			$form->bind($data);

		}
		catch (Exception $e)
		{
			$this->setError($e->getMessage());
			return false;
		}

		$this->forms[$hash] = $form;

		return $form;
	}
	
	protected function preprocessForm(\JForm $form, $data, $group = 'content')
	{		
		\JPluginHelper::importPlugin($group);
	
		$dispatcher = \JEventDispatcher::getInstance();

		$results = $dispatcher->trigger('onContentPrepareForm', array($form, $data));

		if (count($results) && in_array(false, $results, true))
		{			
			$error = $dispatcher->getError();

			if (!($error instanceof Exception))
			{
				throw new Exception($error);
			}
		}		
	}
		
	public function addScript($file, $type = "text/javascript", $defer = false, $async = false)
	{		
		$this->assets->addScript($file, $type, $defer, $async);
	}	

	
	public function addScriptDeclaration($content, $type = 'text/javascript')
	{
		$this->assets->addScriptDeclaration($content, $type);	
	}
	

	public function addStyleSheet($file, $type = 'text/css', $media = null, $attribs = array())
	{		
		$this->assets->addStyleSheet($file, $type, $media, $attribs);
	}

	
	public function addStyleDeclaration($content, $type = 'text/css')
	{
		$this->assets->addStyleDeclaration($content, $type);		
	}
	
	public function __toString() {
		return $this->render();
	}
}
